/* eslint-disable no-param-reassign,prefer-destructuring */
import React, { PureComponent } from 'react';
import { Table, Button } from 'antd';
import { DragDropContext, DragSource, DropTarget } from 'react-dnd';
import HTML5Backend from 'react-dnd-html5-backend';
import styles from './index.less';

class AXDragSortTable extends PureComponent {
  handleMoveRow = (dragIndex, hoverIndex) => {
    const { data, onChange } = this.props;
    const newData = data;
    newData[dragIndex] = newData.splice(hoverIndex, 1, newData[dragIndex])[0];
    onChange(newData);
  };
  swapper = (arr, targetIndex, currentIndex) => {
    const tmpArray = [...arr];
    tmpArray[currentIndex] = tmpArray.splice(targetIndex, 1, tmpArray[currentIndex])[0];
    return tmpArray;
  };
  handleMoveUp = (data, index) => {
    if (index === 0) {
      return;
    }
    this.props.onChange(this.swapper(data, index - 1, index));
  };
  handleMoveDown = (data, index) => {
    if (index === data.length - 1) {
      return;
    }
    this.props.onChange(this.swapper(data, index + 1, index));
  };
  handleRemove = (data, index) => {
    const tmpArray = [...data];
    tmpArray.splice(index, 1);
    this.props.onChange(tmpArray, data[index].id);
  };

  render() {
    const { data, columns } = this.props;

    const addExtraDatas = (originalData) => {
      return (originalData || []).map((item, index) => ({
        ...item,
        key: item.id ? item.id : index + 1,
        _location: index + 1,
      }));
    };

    const addExtraColumns = (newColumns) => {
      const headColumn = {
        title: '位置',
        dataIndex: '_location',
        key: '+01',
        width: 60,
        align: 'center',
      };
      const tailColumn = {
        title: '操作',
        dataIndex: '_operation',
        key: '-01',
        width: 190,
        align: 'right',
        render: (text, record, index) => (
          <div className={styles.operation}>
            <span>
              <Button
                size="small"
                type="primary"
                onClick={() => this.handleMoveUp(data, index)}
              >上移
              </Button>
            </span>
            <span>
              <Button
                size="small"
                type="primary"
                onClick={() => this.handleMoveDown(data, index)}
              >下移
              </Button>
            </span>
            <span>
              <Button
                size="small"
                style={{ background: '#f5222d', color: '#fff' }}
                onClick={() => this.handleRemove(data, index)}
              >删除
              </Button>
            </span>
          </div>
        ),
      };
      return [headColumn, ...newColumns, tailColumn];
    };

    const dragDirection = (
      dragIndex,
      hoverIndex,
      initialClientOffset,
      clientOffset,
      sourceClientOffset,
    ) => {
      const hoverMiddleY = (initialClientOffset.y - sourceClientOffset.y) / 2;
      const hoverClientY = clientOffset.y - sourceClientOffset.y;
      if (dragIndex < hoverIndex && hoverClientY > hoverMiddleY) {
        return 'downward';
      }
      if (dragIndex > hoverIndex && hoverClientY < hoverMiddleY) {
        return 'upward';
      }
    };

    let BodyRow = (props) => {
      const {
        isOver,
        connectDragSource,
        connectDropTarget,
        moveRow,
        dragRow,
        clientOffset,
        sourceClientOffset,
        initialClientOffset,
        ...restProps
      } = props;
      const style = { ...restProps.style, cursor: 'move' };

      let { className } = restProps;
      if (isOver && initialClientOffset) {
        const direction = dragDirection(
          dragRow.index,
          restProps.index,
          initialClientOffset,
          clientOffset,
          sourceClientOffset
        );
        if (direction === 'downward') {
          className += ' drop-over-downward';
        }
        if (direction === 'upward') {
          className += ' drop-over-upward';
        }
      }

      return connectDragSource(
        connectDropTarget(
          <tr
            {...restProps}
            className={className}
            style={style}
          />
        )
      );
    };

    const rowSource = {
      beginDrag(props) {
        return {
          index: props.index,
        };
      },
    };

    const rowTarget = {
      drop(props, monitor) {
        const dragIndex = monitor.getItem().index;
        const hoverIndex = props.index;
        if (dragIndex === hoverIndex) {
          return;
        }
        props.moveRow(dragIndex, hoverIndex);
        monitor.getItem().index = hoverIndex;
      },
    };

    BodyRow = DropTarget('row', rowTarget, (connect, monitor) => ({
      connectDropTarget: connect.dropTarget(),
      isOver: monitor.isOver(),
      sourceClientOffset: monitor.getSourceClientOffset(),
    }))(
      DragSource('row', rowSource, (connect, monitor) => ({
        connectDragSource: connect.dragSource(),
        dragRow: monitor.getItem(),
        clientOffset: monitor.getClientOffset(),
        initialClientOffset: monitor.getInitialClientOffset(),
      }))(BodyRow)
    );

    const components = {
      body: {
        row: BodyRow,
      },
    };

    return (
      <Table
        bordered
        pagination={false}
        className={styles.dragTable}
        columns={addExtraColumns(columns)}
        dataSource={addExtraDatas(data)}
        components={components}
        onRow={(record, index) => ({
          index,
          moveRow: this.handleMoveRow,
        })}
      />
    );
  }
}

export default DragDropContext(HTML5Backend)(AXDragSortTable);
