import React, { Component } from 'react';
import { Row, Col, Card, Table, Radio } from 'antd';
import { genAbsolutePicUrl } from '../../utils/utils';
import styles from './SupportItem.less';

function SupportTpl({ title, subTitle, coverUrl }) {
  return (
    <div className={styles.content}>
      <div className={styles.courseWrapper}>
        <img src={genAbsolutePicUrl(coverUrl)} alt="" />
        <div className={styles.desc}>
          <div className={styles.title}>{title}</div>
          <div className={styles.subTitle}>{subTitle}</div>
        </div>
      </div>
    </div>
  );
}

class SupportItem extends Component {
  state = {
    openKey: 'support',
  };
  handleRadioChange = (e) => {
    this.setState({ openKey: e.target.value });
  };
  render() {
    const { supportList } = this.props;
    const supportColumns = [{
      title: '配套编号',
      dataIndex: 'code',
      key: 1,
      width: '50%',
    }, {
      title: '配套名称',
      dataIndex: 'name',
      key: 2,
      width: '50%',
    }];
    return (
      <Row gutter={16} style={{ marginBottom: 16 }}>
        <Col span={9}>
          <Card title="配套信息" style={{ height: 540 }} className={styles.coverCard}>
            <SupportTpl {...this.props} />
          </Card>
        </Col>
        <Col span={15}>
          <Card
            title={
              <Radio.Group
                value={this.state.openKey}
                onChange={this.handleRadioChange}
              >
                <Radio.Button value="support">相关配套</Radio.Button>
              </Radio.Group>
            }
            style={{ height: 540 }}
          >
            {this.state.openKey === 'support' ? (
              <Table
                pagination={false}
                rowKey={record => record.id}
                dataSource={supportList}
                columns={supportColumns}
                scroll={{ y: 400 }}
                className={styles.dataTable}
              />
            ) : null}
          </Card>
        </Col>
      </Row>
    );
  }
}
export default SupportItem;
