import React, { Fragment } from 'react';
import { Link, Redirect, Switch, Route } from 'dva/router';
import DocumentTitle from 'react-document-title';
import { Icon } from 'antd';
import styles from './UserLayout.less';
import GlobalFooter from '../components/GlobalFooter';
import logo from '../assets/logo.jpg';
import { getRoutes } from '../utils/utils';
import { Hotax } from '../utils/config';

const copyrightNode = <Fragment>Copyright <Icon type="copyright" />{Hotax.CopyRight}</Fragment>;

class UserLayout extends React.PureComponent {
  getPageTitle() {
    const { routerData, location } = this.props;
    const { pathname } = location;
    let title = Hotax.PROJECT_NAME;
    if (routerData[pathname] && routerData[pathname].name) {
      title = `${routerData[pathname].name} - ${Hotax.PROJECT_NAME}`;
    }
    return title;
  }
  render() {
    const { routerData, match } = this.props;
    return (
      <DocumentTitle title={this.getPageTitle()}>
        <div className={styles.container}>
          <div className={styles.content}>
            <div className={styles.top}>
              <div className={styles.header}>
                <Link to="/">
                  <img alt="logo" className={styles.logo} src={logo} />
                  <span className={styles.title}>{Hotax.PROJECT_NAME}</span>
                </Link>
              </div>
              <div className={styles.desc}>双师云课堂后台管理系统</div>
            </div>
            <Switch>
              {getRoutes(match.path, routerData).map(item =>
                (
                  <Route
                    key={item.key}
                    path={item.path}
                    component={item.component}
                    exact={item.exact}
                  />
                )
              )}
              <Redirect exact from="/user" to="/user/login" />
            </Switch>
          </div>
          <GlobalFooter colorInverse className={styles.footer} copyright={copyrightNode} />
        </div>
      </DocumentTitle>
    );
  }
}

export default UserLayout;
