import { message } from 'antd';
import { routerRedux } from 'dva/router';
import {
  queryMerchantList,
  queryMerchantItem,
  createMerchantItem,
  updateMerchantItem,
  deleteMerchantItem,
  depositMerchantItem,
  queryMerchantRecommend,
  updateMerchantRecommend,
  queryMerchantPoster,
  createMerchantPosterItem,
  updateMerchantPosterItem,
  deleteMerchantPosterItem, queryMerchantPosterItem,
} from '../services/merchant';
import { Hotax } from '../utils/config';

export default {
  namespace: 'merchant',

  state: {
    list: [],
    pageNo: 1,
    pageSize: 15,
    totalSize: 0,
    currentItem: {},
    recommendList: [],
  },

  effects: {
    *fetchMerchantList({ payload }, { call, put }) {
      const response = yield call(queryMerchantList, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: response.data.list || [],
            pageSize: response.data.pageSize,
            totalSize: response.data.totalSize,
            pageNo: response.data.pageNo,
          },
        });
      }
    },
    *fetchMerchantItem({ payload }, { call, put }) {
      const response = yield call(queryMerchantItem, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            currentItem: response.data || {},
          },
        });
      }
    },
    *createMerchantItem({ payload, state }, { call, put }) {
      const response = yield call(createMerchantItem, payload);
      if (response.success) {
        message.success('创建商户成功');
        yield put(routerRedux.push({
          state,
          pathname: '/merchant/list',
        }));
      }
    },
    *deleteMerchantItem({ payload, states }, { call, put }) {
      const response = yield call(deleteMerchantItem, payload);
      if (response.success) {
        message.success('删除商户成功');
        yield put({
          type: 'fetchMerchantList',
          payload: states.Queryers,
        });
      }
    },
    *updateMerchantItem({ payload, states }, { call, put }) {
      const response = yield call(updateMerchantItem, payload);
      if (response.success) {
        message.success('修改商户成功');
        yield put(routerRedux.push({
          pathname: '/merchant/list',
          state: states,
        }));
      }
    },
    *depositMerchantItem({ payload, states }, { call, put }) {
      const response = yield call(depositMerchantItem, payload);
      if (response.success) {
        message.success('账户充值成功');
        yield put(routerRedux.push({
          pathname: '/merchant/list',
          state: states,
        }));
      }
    },
    *queryMerchantRecommend({ payload }, { call, put }) {
      const response = yield call(queryMerchantRecommend, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            recommendList: response.data || [],
          },
        });
      }
    },
    *updateMerchantRecommend({ payload, states }, { call, put }) {
      const response = yield call(updateMerchantRecommend, payload);
      if (response.success) {
        message.success('推荐课程列表更新成功');
        yield put(routerRedux.push({
          pathname: '/frontend/recommend',
          state: states,
        }));
      }
    },
    *fetchMerchantPoster({ payload }, { call, put }) {
      const response = yield call(queryMerchantPoster, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            posterList: response.data || [],
          },
        });
      }
    },
    *fetchMerchantPosterItem({ payload }, { call, put }) {
      const response = yield call(queryMerchantPosterItem, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            posterItem: response.data || {},
          },
        });
      }
    },
    *createMerchantPosterItem({ payload, posterId }, { call, put, select }) {
      const response = yield call(createMerchantPosterItem, payload);
      if (response.success) {
        message.success('创建海报成功');
        // 创建成功，更新该条数据id
        const originalData = yield select(state => state.merchant.posterList);
        const newData = originalData.map((data) => {
          const { id, isNew, isEdit, ...rest } = data;
          if (id === posterId) {
            return { ...rest, id: response.data.id };
          }
          return { ...data };
        });
        yield put({
          type: 'fixPosterList',
          payload: newData,
        });
      }
    },
    *updateMerchantPosterItem({ payload, posterId }, { call, put, select }) {
      const response = yield call(updateMerchantPosterItem, payload);
      if (response.success) {
        message.success('修改海报成功');
        // 修改成功，去掉isEdit字段
        const originalData = yield select(state => state.merchant.posterList);
        const newData = originalData.map((data) => {
          const { isEdit, ...rest } = data;
          if (data.id === posterId) {
            return { ...rest };
          }
          return { ...data };
        });
        yield put({
          type: 'fixPosterList',
          payload: newData,
        });
      }
    },
    *deleteMerchantPosterItem({ payload }, { call, put, select }) {
      const response = yield call(deleteMerchantPosterItem, payload);
      if (response.success) {
        message.success('删除海报成功');
        // 删除成功后修改该条的status字段为删除状态
        const { posterId } = payload;
        const originalData = yield select(state => state.merchant.posterList);
        const newData = originalData.map((data) => {
          if (data.id === posterId) {
            return { ...data, status: Hotax.STATUS_DELETE };
          }
          return { ...data };
        });
        yield put({
          type: 'fixPosterList',
          payload: newData,
        });
      }
    },
  },

  reducers: {
    querySuccess(state, action) {
      return {
        ...state,
        ...action.payload,
      };
    },
    fixRecommendList(state, action) {
      return {
        ...state,
        recommendList: action.payload,
      };
    },
    fixPosterList(state, action) {
      return {
        ...state,
        posterList: action.payload,
      };
    },
    cleanItemState(state) {
      return {
        ...state,
        currentItem: {},
      };
    },
  },
};
