import { message } from 'antd';
import { routerRedux } from 'dva/router';
import {
  queryTagList,
  queryTagItem,
  createTagItem,
  updateTagItem,
  deleteTagItem,
} from '../services/tag';

export default {
  namespace: 'tag',

  state: {
    list: [],
    pageNo: 1,
    pageSize: 15,
    totalSize: 0,
    currentItem: {},
  },

  effects: {
    *fetchTagList({ payload }, { call, put }) {
      const response = yield call(queryTagList, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: response.data.list || [],
            pageSize: response.data.pageSize,
            totalSize: response.data.totalSize,
            pageNo: response.data.pageNo,
          },
        });
      }
    },
    *fetchTagItem({ payload }, { call, put }) {
      const response = yield call(queryTagItem, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            currentItem: response.data || {},
          },
        });
      }
    },
    *createTagItem({ payload, states }, { call, put }) {
      const response = yield call(createTagItem, payload);
      if (response.success) {
        message.success('创建标签成功');
        yield put(routerRedux.push({
          state: states,
          pathname: '/frontend/tag',
        }));
      }
    },
    *deleteTagItem({ payload, states }, { call, put }) {
      const response = yield call(deleteTagItem, payload);
      if (response.success) {
        message.success('删除标签成功');
        yield put({
          type: 'fetchTagList',
          payload: states.Queryers,
        });
      }
    },
    *updateTagItem({ payload, states }, { call, put }) {
      const response = yield call(updateTagItem, payload);
      if (response.success) {
        message.success('修改标签成功');
        yield put(routerRedux.push({
          pathname: '/frontend/tag',
          state: states,
        }));
      }
    },
  },

  reducers: {
    querySuccess(state, action) {
      return {
        ...state,
        ...action.payload,
      };
    },
    fixCurrentItem(state, action) {
      return {
        ...state,
        currentItem: {
          ...state.currentItem,
          ...action.payload,
        },
      };
    },
    cleanItemState(state) {
      return {
        ...state,
        currentItem: {},
      };
    },
  },
};
