import React, { PureComponent } from 'react';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Input, Button, message } from 'antd';
import Uploader from '../../../components/AXUpload/index';
import styles from './PictureMultipleUpload.less';

const Message = message;

@connect(({ loading }) => ({
  submitting: loading.models.resource,
}))
export default class PictureMultipleUpload extends PureComponent {
  state = {
    fileList: [],
  };

  handleUploadSuccess = (fileList) => {
    this.setState({ fileList });
  };
  handleOnRemove = (fileList) => {
    this.setState({ fileList });
  };
  handleCodeInputChange = (index, e) => {
    const newFileList = [...this.state.fileList];
    newFileList[index].code = e.target.value;
    this.setState({
      fileList: newFileList,
    });
  };
  handleNameInputChange = (index, e) => {
    const newFileList = [...this.state.fileList];
    newFileList[index].name = e.target.value;
    this.setState({
      fileList: newFileList,
    });
  };
  handlePageBack = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/resource/image',
    }));
  };
  handlePageSubmit = () => {
    const { fileList } = this.state;
    let correct = true;
    fileList.forEach((file) => {
      if (!file.name || !file.code) {
        correct = false;
      }
    });
    if (correct) {
      const payload = fileList.map(file => ({
        name: file.name,
        code: file.code,
        format: file.type,
        size: file.size,
        path: file.path,
        url: file.url,
        status: 'NORMAL',
      }));
      this.props.dispatch({
        payload,
        type: 'resource/createImage',
      });
    } else {
      Message.error('还有图片编号或名称未指定!');
    }
  };

  render() {
    const { fileList } = this.state;
    const { submitting } = this.props;
    const renderPictureCard = (data, index) => {
      return (
        <div key={`item-${index}`} className={styles.itemWrapper}>
          <div className={styles.item}>
            <div className={styles.imgWrapper}>
              <img src={data.url} alt="uploaded" />
            </div>
          </div>
          <div className={styles.inputWrapper}>
            <Input
              value={data.code}
              style={{ width: '100%', marginTop: 2 }}
              placeholder="请输入图片编号"
              onChange={e => this.handleCodeInputChange(index, e)}
            />
            <Input
              value={data.name && data.name.split('.')[0]}
              style={{ width: '100%', marginTop: 2 }}
              placeholder="请输入图片名称"
              onChange={e => this.handleNameInputChange(index, e)}
            />
          </div>
        </div>
      );
    };
    return (
      <div className={styles.content}>
        <div className={styles.left}>
          <Uploader
            forbidden={false}
            fileList={fileList}
            accept="image"
            multiple
            totalLimit={20}
            onUpload={this.handleUploadSuccess}
            onRemove={this.handleOnRemove}
          />
        </div>
        <div className={styles.right}>
          {fileList.map((item, index) => renderPictureCard(item, index))}
        </div>
        <div className={styles.bottom}>
          <Button onClick={this.handlePageBack}>取消</Button>
          <Button
            type="primary"
            style={{ marginLeft: 10 }}
            loading={submitting}
            onClick={this.handlePageSubmit}
          >提交
          </Button>
        </div>
      </div>
    );
  }
}
