import React, { Component } from 'react';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Button, Icon, Card } from 'antd';
import Result from '../../../components/Result';
import { genAbsolutePicUrl } from '../../../utils/utils';

class PictureUploadSuccessResult extends Component {
  jumpToPictureListPage = () => {
    this.props.dispatch(
      routerRedux.push({
        pathname: '/resource/picture',
      })
    );
  };
  jumpToPictureDetailPage = (params) => {
    this.props.dispatch(
      routerRedux.push({
        pathname: `/resource/picture-edit/:${params.id}`,
        state: params,
      })
    );
  };
  jumpToPictureSingleUpload = () => {
    this.props.dispatch(
      routerRedux.push({
        pathname: '/resource/picture-upload/single',
      })
    );
  };
  render() {
    const { location } = this.props;
    const { state = {} } = location;
    const { path, code, format, name, size } = state;
    const description = '图片上传成功，请核对以下图片信息，如果有误请编辑图片进行修改';
    const extra = (
      <div>
        <div style={{ fontSize: 16, color: 'rgba(0, 0, 0, 0.85)', fontWeight: 500, marginBottom: 16 }}>
          您提交的图片信息如下:
        </div>
        <div style={{ marginBottom: 16 }}>
          <Icon style={{ color: '#52c41a', marginRight: 8 }} type="check-circle-o" />图片名称:
          <span style={{ marginLeft: 16 }}>{name || ''}</span>
        </div>
        <div style={{ marginBottom: 16 }}>
          <Icon style={{ color: '#52c41a', marginRight: 8 }} type="check-circle-o" />图片编号:
          <span style={{ marginLeft: 16 }}>{code || ''}</span>
        </div>
        <div style={{ marginBottom: 16 }}>
          <Icon style={{ color: '#52c41a', marginRight: 8 }} type="check-circle-o" />图片格式:
          <span style={{ marginLeft: 16 }}>{format || ''}</span>
        </div>
        <div style={{ marginBottom: 16 }}>
          <Icon style={{ color: '#52c41a', marginRight: 8 }} type="check-circle-o" />图片大小:
          <span style={{ marginLeft: 16 }}>{`${size || ''}字节`}</span>
        </div>
        <div style={{ marginBottom: 16 }}>
          <Icon style={{ color: '#52c41a', marginRight: 8 }} type="check-circle-o" />图片地址:
          <span style={{ marginLeft: 16 }}>{path ? genAbsolutePicUrl(path) : ''}</span>
        </div>
      </div>
    );
    const actions = (
      <div>
        <Button
          onClick={this.jumpToPictureListPage}
        >返回列表
        </Button>
        <Button
          type="primary"
          style={{ marginLeft: 10 }}
          onClick={this.jumpToPictureSingleUpload}
        >再传一个
        </Button>
        <Button
          type="primary"
          style={{ marginLeft: 10 }}
          onClick={() => this.jumpToPictureDetailPage(state)}
        >修改图片
        </Button>
      </div>
    );
    return (
      <Card>
        <Result
          type="success"
          title="上传成功"
          description={description}
          extra={extra}
          actions={actions}
        />
      </Card>
    );
  }
}

export default connect()(PictureUploadSuccessResult);
