/* eslint-disable no-trailing-spaces */
import React, { PureComponent } from 'react';
import { message, Card, Modal, List, Form, Input, Button, Radio, Popover, Icon, Switch } from 'antd';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import Selector from '../../../components/AXTableSelector/Selector';
import FooterToolbar from '../../../components/FooterToolbar';
import { boolToStatus } from '../../../utils/utils';
import styles from './CmsUserCreate.less';

const Message = message;

const genders = [{
  title: '男',
  value: 'MALE',
}, {
  title: '女',
  value: 'FEMALE',
}];
const fieldLabels = {
  campus: '所属商户',
  name: '用户名称',
  nickName: '用户昵称',
  gender: '用户性别',
  birthday: '出生日期',
  mobile: '电话',
  mail: '邮箱',
  qq: 'QQ号码',
  weChat: '微信号码',
  password: '用户密码',
  confirm: '确认密码',
  status: '初始状态',
};
const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 6 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 14 },
    md: { span: 12 },
  },
};

@Form.create()
@connect(({ loading, merchant, cmsUser }) => ({
  merchant,
  cmsUser,
  mLoading: loading.models.merchant,
  submitting: loading.models.cmsUser,
}))
export default class CmsUserCreatePage extends PureComponent {
  state = {
    merchantSelectorDestroy: true,
  };
  componentWillUnmount() {
    this.props.dispatch({
      type: 'cmsUser/cleanState',
    });
  }
  checkPassword = (rule, value, callback) => {
    if (value && value !== this.props.form.getFieldValue('password')) {
      callback('两次输入的密码不一致');
    } else {
      callback();
    }
  }
  handleMerchantSelectorModalShow = () => {
    this.setState({
      merchantSelectorDestroy: false,
    });
    this.props.dispatch({
      type: 'merchant/fetchMerchantList',
      payload: {},
    });
  }
  handleMerchantSelectorFinish = (rows) => {
    this.setState({
      merchantSelectorDestroy: true,
    });
    if (!rows || !rows.length) {
      return;
    }
    const { id, code, name } = rows[0];
    this.props.dispatch({
      type: 'cmsUser/fixCurrentItem',
      payload: {
        code,
        merchantId: id,
        merchantName: name,
      },
    });
  };
  handleMerchantSelectorCancel = () => {
    this.setState({
      merchantSelectorDestroy: true,
    });
  };
  handleMerchantSelectorChange = (params) => {
    this.props.dispatch({
      type: 'merchant/fetchMerchantList',
      payload: params,
    });
  }
  handlePageSubmit = () => {
    this.props.form.validateFieldsAndScroll((error, values) => {
      const { currentItem } = this.props.cmsUser;
      const { merchantId } = currentItem;
      if (!merchantId) {
        Message.error('请选择厂商');
        return;
      }
      const { status, ...restProps } = values;
      restProps.status = boolToStatus(status);
      restProps.merchantId = merchantId;
      if (!error) {
        this.props.dispatch({
          type: 'cmsUser/createCmsUserItem',
          payload: restProps,
          states: this.props.location.state,
        });
      }
    });
  }
  handlePageBack = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/system/cms-user/list',
      state: this.props.location.state,
    }));
  }

  render() {
    const { merchantSelectorDestroy } = this.state;
    const { form, mLoading, submitting, merchant, cmsUser } = this.props;
    const { getFieldDecorator, getFieldsError } = form;
    const { currentItem } = cmsUser;
    const { code, merchantName } = currentItem;

    const errors = getFieldsError();
    const getErrorInfo = () => {
      const errorCount = Object.keys(errors).filter(key => errors[key]).length;
      if (!errors || errorCount === 0) {
        return null;
      }
      const scrollToField = (fieldKey) => {
        const labelNode = document.querySelector(`label[for="${fieldKey}"]`);
        if (labelNode) {
          labelNode.scrollIntoView(true);
        }
      };
      const errorList = Object.keys(errors).map((key) => {
        if (!errors[key]) {
          return null;
        }
        return (
          <li key={key} className={styles.errorListItem} onClick={() => scrollToField(key)}>
            <Icon type="cross-circle-o" className={styles.errorIcon} />
            <div className={styles.errorMessage}>{errors[key][0]}</div>
            <div className={styles.errorField}>{fieldLabels[key]}</div>
          </li>
        );
      });
      return (
        <span className={styles.errorIcon}>
          <Popover
            title="表单校验信息"
            content={errorList}
            overlayClassName={styles.errorPopover}
            trigger="click"
            getPopupContainer={trigger => trigger.parentNode}
          >
            <Icon type="exclamation-circle" />
          </Popover>
          {errorCount}
        </span>
      );
    };

    const getMerchantModal = () => {
      return (
        <Modal
          visible
          width={1100}
          footer={null}
          title="厂商列表"
          maskClosable={false}
          onCancel={this.handleMerchantSelectorCancel}
        >
          <Selector
            multiple={false}
            loading={mLoading}
            selectorName="Merchant"
            list={merchant.list}
            pageNo={merchant.pageNo}
            pageSize={merchant.pageSize}
            totalSize={merchant.totalSize}
            onCancel={this.handleMerchantSelectorCancel}
            onChange={this.handleMerchantSelectorChange}
            onFinish={this.handleMerchantSelectorFinish}
          />
        </Modal>
      );
    };

    return (
      <div>
        <Card title="创建用户" style={{ marginBottom: 70 }}>
          <Form>
            <Form.Item
              {...formItemLayout}
              label={<Button size="small" type="primary" onClick={this.handleMerchantSelectorModalShow}>选择厂商</Button>}
            >
              <List
                bordered
                size="small"
                dataSource={[
                  `厂商编号: ${code || ''}`,
                  `厂商名称: ${merchantName || ''}`,
                ]}
                renderItem={item => <List.Item>{item}</List.Item>}
              />
            </Form.Item>
            <Form.Item hasFeedback label={fieldLabels.name} {...formItemLayout}>
              {getFieldDecorator('name', {
                rules: [{ required: true, message: '请填写用户名' }],
              })(
                <Input placeholder="请输入" />
              )}
            </Form.Item>
            <Form.Item hasFeedback label={fieldLabels.nickName} {...formItemLayout}>
              {getFieldDecorator('nickName', {
                rules: [{ required: true, message: '请填写用户昵称' }],
              })(
                <Input placeholder="请输入" />
              )}
            </Form.Item>
            <Form.Item hasFeedback label={fieldLabels.password} {...formItemLayout}>
              {getFieldDecorator('password', {
                rules: [
                  {
                    required: true, message: '密码不能为空!',
                  }, {
                    pattern: /^[a-zA-Z0-9|-]+$/ig, message: '密码格式错误!',
                  }],
              })(
                <Input
                  type="password"
                  placeholder="请输入"
                  addonBefore={<Icon type="lock" />}
                />
              )}
            </Form.Item>
            <Form.Item hasFeedback label={fieldLabels.confirm} {...formItemLayout}>
              {getFieldDecorator('confirm', {
                rules: [
                  {
                    required: true, message: '请确认密码',
                  }, {
                    validator: this.checkPassword,
                  }],
              })(
                <Input
                  type="password"
                  placeholder="请输入"
                  addonBefore={<Icon type="lock" />}
                />
              )}
            </Form.Item>
            <Form.Item label={fieldLabels.birthday} {...formItemLayout}>
              {getFieldDecorator('birthday')(
                <Input placeholder="请输入" />
              )}
            </Form.Item>
            <Form.Item label={fieldLabels.mobile} {...formItemLayout}>
              {getFieldDecorator('mobile', {
                rules: [{ pattern: /^[1][34578][0-9]{9}$/g, message: '请输入11位有效手机号!' }],
              })(
                <Input placeholder="请输入" />
              )}
            </Form.Item>
            <Form.Item label={fieldLabels.mail} {...formItemLayout}>
              {getFieldDecorator('mail')(
                <Input placeholder="请输入" />
              )}
            </Form.Item>
            <Form.Item label={fieldLabels.qq} {...formItemLayout}>
              {getFieldDecorator('qq')(
                <Input placeholder="请输入" />
              )}
            </Form.Item>
            <Form.Item label={fieldLabels.weChat} {...formItemLayout}>
              {getFieldDecorator('weChat')(
                <Input placeholder="请输入" />
              )}
            </Form.Item>
            <Form.Item label={fieldLabels.gender} {...formItemLayout}>
              {getFieldDecorator('gender', {
                initialValue: 'MALE',
              })(
                <Radio.Group className={styles.radio}>
                  {
                    genders.map(item =>
                      (
                        <Radio.Button
                          key={item.value}
                          value={item.value}
                        >{item.title}
                        </Radio.Button>
                      )
                    )
                  }
                </Radio.Group>
              )}
            </Form.Item>
            <Form.Item label={fieldLabels.status} {...formItemLayout}>
              {getFieldDecorator('status', {
                valuePropName: 'checked',
                initialValue: true,
              })(
                <Switch
                  checkedChildren="启用"
                  unCheckedChildren="禁用"
                />
              )}
            </Form.Item>
          </Form>
          {!merchantSelectorDestroy && getMerchantModal()}
        </Card>
        <FooterToolbar style={{ width: '100%' }}>
          {getErrorInfo()}
          <Button
            onClick={this.handlePageBack}
            style={{ marginRight: 10 }}
          >取消
          </Button>
          <Button
            type="primary"
            loading={submitting}
            onClick={this.handlePageSubmit}
          >提交
          </Button>
        </FooterToolbar>
      </div>
    );
  }
}
