/* eslint-disable prefer-destructuring */
import React, { Component } from 'react';
import moment from 'moment';
import { connect } from 'dva';
import { Modal, Form, Card, Badge, Button, DatePicker, message } from 'antd';
import { StandardTableList } from '../../../components/AXList';
import Ellipsis from '../../../components/Ellipsis';
import { addRowKey } from '../../../utils/utils';
import styles from './AuthList.less';

const Message = message;

const formItemLayout = {
  labelCol: {
    md: { span: 6 },
  },
  wrapperCol: {
    md: { span: 18 },
  },
};
const timestamp2Str = ts => moment(ts).format('YYYY-MM-DD HH:mm:ss');

@Form.create()
@connect(({ terminal, loading }) => ({
  terminal,
  loading: loading.models.terminal,
}))
export default class TerminalAuthListPage extends Component {
  constructor(props) {
    super(props);
    const { state } = props.location;
    this.state = {
      UIParams: (state || {}).UIParams, // 组件的状态参数
      Queryers: (state || {}).Queryers, // 查询的条件参数
      authEditModalDestroy: true,
      targetRow: {},
    };
  }
  componentWillMount() {
    this.props.dispatch({
      type: 'terminal/cleanState',
    });
  }
  componentDidMount() {
    this.props.dispatch({
      type: 'terminal/fetchTerminalAuthList',
      payload: { ...this.state.Queryers },
    });
  }
  handleFilterOperation = (params, states) => {
    this.props.dispatch({
      type: 'terminal/fetchTerminalAuthList',
      payload: params,
    });
    this.setState({
      UIParams: states,
      Queryers: params,
    });
  };
  handleBatchOperation = () => {
    Message.info('暂不支持批量操作!');
  };
  handleEditAuthModalShow = (record) => {
    this.setState({
      authEditModalDestroy: false,
      targetRow: record,
    });
  };
  handleEditAuthModalHide = () => {
    this.setState({
      authEditModalDestroy: true,
    });
  };
  handleEditAuthModalFinish = () => {
    this.props.form.validateFieldsAndScroll((err, values) => {
      if (!err) {
        const { startTime, endTime } = values;
        const stStr = moment(startTime).format('YYYY-MM-DD');
        const etStr = moment(endTime).format('YYYY-MM-DD');
        const { targetRow } = this.state;
        this.props.dispatch({
          type: 'terminal/updateTerminalAuth',
          payload: { ...targetRow, startTime: stStr, endTime: etStr },
          states: { ...this.state.Queryers },
        });
      }
    });
    this.setState({
      authEditModalDestroy: true,
    });
  };
  render() {
    const { targetRow, authEditModalDestroy } = this.state;
    const { terminal, loading, form } = this.props;
    const { getFieldDecorator } = form;
    const { pageNo, pageSize, totalSize, list } = terminal;
    const basicSearch = {
      keys: [{
        name: '终端编号',
        field: 'code',
      }],
    };
    const batchActions = [{
      key: 'delete',
      name: '批量删除',
    }];
    const pagination = {
      pageNo,
      pageSize,
      totalSize,
    };
    const columns = [{
      title: '终端编号',
      key: 1,
      dataIndex: 'ucode',
      width: '15%',
      render: text => (
        <Ellipsis tooltip lines={1}>{text}</Ellipsis>
      ),
    }, {
      title: '产品编号',
      key: 2,
      dataIndex: 'pcode',
      width: '12%',
      render: text => (
        <Ellipsis tooltip lines={1}>{text}</Ellipsis>
      ),
    }, {
      title: '产品名称',
      key: 3,
      dataIndex: 'pname',
      width: '13%',
      render: text => (
        <Ellipsis tooltip lines={1}>{text}</Ellipsis>
      ),
    }, {
      title: '权限有效期',
      key: 4,
      render: (_, record) => {
        const { startTime, endTime } = record;
        return (
          <div className={styles.authDesc}>
            <p><span>起始时间:&nbsp;&nbsp;</span>{`${timestamp2Str(startTime)}`}</p>
            <p><span>到期时间:&nbsp;&nbsp;</span>{`${timestamp2Str(endTime)}`}</p>
          </div>
        );
      },
      width: '25%',
      align: 'center',
    }, {
      title: '权限有效时长',
      key: 5,
      dataIndex: 'endTime',
      render: (text) => {
        const day = moment(text).diff(moment(), 'days');
        if (day < 0) {
          return <Badge status="error" text="已到期" />;
        }
        return <span><span style={{ color: '#52c41a', fontWeight: 'bold' }}>{day}</span>天到期</span>;
      },
      width: '12%',
      align: 'center',
    }, {
      title: '修改时间',
      key: 6,
      dataIndex: 'updateTime',
      render: text => timestamp2Str(text),
      width: '15%',
      align: 'center',
    }, {
      title: '操作',
      key: 7,
      render: (_, record) => (
        <Button onClick={() => this.handleEditAuthModalShow(record)} size="small" className="editBtn">修改权限</Button>
      ),
      width: '8%',
      align: 'right',
    }];
    return (
      <Card>
        <StandardTableList
          columns={columns}
          loading={loading}
          dataSource={addRowKey(list)}
          header={{
            basicSearch,
            onFilterClick: this.handleFilterOperation,
          }}
          footer={{
            pagination,
            batchActions,
            onBatchClick: this.handleBatchOperation,
          }}
          keepUIState={{ ...this.state.UIParams }}
          showStatusSelect={false}
        />
        {!authEditModalDestroy && (
          <Modal
            visible
            title="修改权限时间"
            cancelText="取消"
            okText="确定"
            maskClosable={false}
            onCancel={this.handleEditAuthModalHide}
            onOk={this.handleEditAuthModalFinish}
          >
            <Form>
              <Form.Item label="起始日期" {...formItemLayout}>
                {getFieldDecorator('startTime', {
                  rules: [{ required: true, message: '请选择起始日期' }],
                  initialValue: targetRow && targetRow.startTime && moment(targetRow.startTime),
                })(
                  <DatePicker
                    showTime
                    placeholder="选择日期时间"
                    format="YYYY-MM-DD HH:mm:ss"
                    style={{ width: 280 }}
                  />
                )}
              </Form.Item>
              <Form.Item label="截止日期" {...formItemLayout}>
                {getFieldDecorator('endTime', {
                  rules: [{ required: true, message: '请选择结束日期' }],
                  initialValue: targetRow && targetRow.endTime && moment(targetRow.endTime),
                })(
                  <DatePicker
                    showTime
                    placeholder="选择日期时间"
                    format="YYYY-MM-DD HH:mm:ss"
                    style={{ width: 280 }}
                  />
                )}
              </Form.Item>
            </Form>
          </Modal>
        )}
      </Card>
    );
  }
}
